﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Microsoft.Rtc.Collaboration;
using Microsoft.Rtc.Signaling;

namespace UCMAAsyncCTP
{
    public static class Extensions
    {
        //TODO: 3.1.3 - Create a Task that represents a pair of begin and end methods
        public static Task StartUpAsync(this CollaborationPlatform platform)
        {
            if (platform == null)
            {
                throw new ArgumentNullException();
            }
   
            return Task.Factory.FromAsync(platform.BeginStartup, platform.EndStartup, null);
        }

        public static Task ShutDownAsync(this CollaborationPlatform platform)
        {
            if (platform == null)
            {
                throw new ArgumentNullException();
            }

            return Task.Factory.FromAsync(platform.BeginShutdown, platform.EndShutdown, null);
        }

        public static Task<SipResponseData> EstablishAsync(this LocalEndpoint endpoint)
        {
            if (endpoint == null)
            {
                throw new ArgumentNullException();
            }

            return Task<SipResponseData>.Factory.FromAsync(endpoint.BeginEstablish, endpoint.EndEstablish, null);
        }

        public static Task TerminateAsync(this LocalEndpoint endpoint)
        {
            if (endpoint == null)
            {
                throw new ArgumentNullException();
            }

            return Task.Factory.FromAsync(endpoint.BeginTerminate, endpoint.EndTerminate, null);
        }

        //TODO: 3.1.5 - Create a Task that represents establishing an instant message call
        public static Task<CallMessageData> EstablishAsync(this Call call, string destinationUri, CallEstablishOptions options)
        {
            if (call == null)
            {
                throw new ArgumentNullException();
            }

            return Task<CallMessageData>.Factory.FromAsync(call.BeginEstablish, call.EndEstablish, destinationUri, options, null);
        }

        public static Task TerminateAsync(this Call call)
        {
            if (call == null)
            {
                throw new ArgumentNullException();
            }

            return Task.Factory.FromAsync(call.BeginTerminate, call.EndTerminate, null);
        }

        public static Task<SendInstantMessageResult> SendInstantMessageAsync(this InstantMessagingFlow flow, string textBody)
        {
            if (flow == null)
            {
                throw new ArgumentNullException();
            }

            return Task<SendInstantMessageResult>.Factory.FromAsync(flow.BeginSendInstantMessage, flow.EndSendInstantMessage, textBody, null);
        }
    }
}
