﻿using System;
using System.Configuration;
using System.Threading;

using Microsoft.Rtc.Collaboration;

namespace UCMAAsyncCTP
{
    public class UCMAAsyncSample
    {
        CollaborationPlatform _platform;
        ApplicationEndpoint _endpoint;

        private static string _applicationId;
        private static string _applicationName;
        private static string _applicationEndpointURI;
        private static string _primaryLabUserId;

        private static string _lyncServer = "w15-lync-se1.contoso.com"; 
        private static int _lyncServerPort = 5061; 

        async internal void Start()
        {
            if (String.IsNullOrEmpty(ConfigurationManager.AppSettings["ApplicationId"]) ||
                (String.IsNullOrEmpty(ConfigurationManager.AppSettings["ApplicationName"])) ||
                (String.IsNullOrEmpty(ConfigurationManager.AppSettings["ApplicationEndpointURI"])) ||
                (String.IsNullOrEmpty(ConfigurationManager.AppSettings["PrimaryLabUserId"])))
            {
                throw new ArgumentNullException("Please provide values for ApplicationId, ApplicationName, ApplicationEndpointURI, and PrimaryLabUserId in app.config.");
            }

            _applicationId = ConfigurationManager.AppSettings["ApplicationId"];
            _applicationName = ConfigurationManager.AppSettings["ApplicationName"];
            _applicationEndpointURI = ConfigurationManager.AppSettings["ApplicationEndpointURI"];
            _primaryLabUserId = ConfigurationManager.AppSettings["PrimaryLabUserId"];

            var platformSettings = new ProvisionedApplicationPlatformSettings(_applicationName, _applicationId);
            _platform = new CollaborationPlatform(platformSettings);

            //TODO: 3.1.2 - Start the CollaborationPlatform
            await _platform.StartUpAsync();

            Console.WriteLine("Platform started.");

            var endpointSettings = new ApplicationEndpointSettings(_applicationEndpointURI, _lyncServer, _lyncServerPort);
            _endpoint = new ApplicationEndpoint(_platform, endpointSettings);

            await _endpoint.EstablishAsync();

            Console.WriteLine("Endpoint started.");

            var conversation = new Conversation(_endpoint);
            var imCall = new InstantMessagingCall(conversation); 
            //TODO: 3.1.4 - Establish an instant message call
            await imCall.EstablishAsync(_primaryLabUserId, null);

            Console.WriteLine("IM call established");

            await imCall.Flow.SendInstantMessageAsync("Hello world!");
            Console.WriteLine("Sent IM.");

            Console.WriteLine("Sleeping for 10 seconds then terminating IM conversation.");
            Thread.Sleep(10000);

            await imCall.TerminateAsync();
            Console.WriteLine("IM call terminated");
        }

        async internal void Stop()
        {
            await _endpoint.TerminateAsync();
            Console.WriteLine("Endpoint terminated.");

            await _platform.ShutDownAsync();
            Console.WriteLine("Platform shut down.");
        }
    }
}
