﻿using System;
using System.Threading.Tasks;
using Microsoft.Rtc.Collaboration;
using Microsoft.Rtc.Signaling;

namespace UCMACoreWorkflow
{
    public static class Extensions
    {
        public static Task StartUpAsync(this CollaborationPlatform platform)
        {
            if (platform == null)
            {
                throw new ArgumentNullException();
            }

            return Task.Factory.FromAsync(platform.BeginStartup, platform.EndStartup, null);
        }

        public static Task ShutDownAsync(this CollaborationPlatform platform)
        {
            if (platform == null)
            {
                throw new ArgumentNullException();
            }

            return Task.Factory.FromAsync(platform.BeginShutdown, platform.EndShutdown, null);
        }

        public static Task<SipResponseData> EstablishAsync(this LocalEndpoint endpoint)
        {
            if (endpoint == null)
            {
                throw new ArgumentNullException();
            }

            return Task<SipResponseData>.Factory.FromAsync(endpoint.BeginEstablish, endpoint.EndEstablish, null);
        }

        public static Task TerminateAsync(this LocalEndpoint endpoint)
        {
            if (endpoint == null)
            {
                throw new ArgumentNullException();
            }

            return Task.Factory.FromAsync(endpoint.BeginTerminate, endpoint.EndTerminate, null);
        }

        public static Task<CallMessageData> EstablishAsync(this Call call, string destinationUri, CallEstablishOptions options)
        {
            if (call == null)
            {
                throw new ArgumentNullException();
            }

            return Task<CallMessageData>.Factory.FromAsync(call.BeginEstablish, call.EndEstablish, destinationUri, options, null);
        }

        public static Task<CallMessageData> AcceptAsync(this Call call, CallAcceptOptions options)
        {
            if (call == null)
            {
                throw new ArgumentNullException();
            }

            return Task<CallMessageData>.Factory.FromAsync(call.BeginAccept, call.EndAccept, options, null);
        }

        public static Task TerminateAsync(this Call call)
        {
            if (call == null)
            {
                throw new ArgumentNullException();
            }

            return Task.Factory.FromAsync(call.BeginTerminate, call.EndTerminate, null);
        }

        public static Task<SendInstantMessageResult> SendInstantMessageAsync(this InstantMessagingFlow flow, string textBody)
        {
            if (flow == null)
            {
                throw new ArgumentNullException();
            }

            return Task<SendInstantMessageResult>.Factory.FromAsync(flow.BeginSendInstantMessage, flow.EndSendInstantMessage, textBody, null);
        }
    }
}
