﻿using System;
using Microsoft.Rtc.Collaboration.AudioVideo;
using Microsoft.Rtc.Signaling;
using Microsoft.Rtc.Collaboration;
using Microsoft.Speech.Synthesis;
using Microsoft.Speech.AudioFormat;

namespace UCMACoreWorkflow
{
    class PhoneMenuActivity2Handler
    {
        public static CallController PhoneMenuCallController = CallController.Instance;

        async internal void Start()
        {
            try
            {
                PhoneMenuCallController.SpeechSynthesizer.SpeakCompleted
                    += new EventHandler<SpeakCompletedEventArgs>(SpeechSynth_SpeakCompleted);
                PhoneMenuCallController.ToneController.ToneReceived
                    += new EventHandler<ToneControllerEventArgs>(ToneController_ToneReceived);

                PlayMenu(MenuType.Initial);
            }
            catch (InvalidOperationException ex)
            {
                Console.WriteLine(ex);
            }
            catch (RealTimeException ex)
            {
                Console.WriteLine(ex);
            }
        }

        void PlayMenu(MenuType type)
        {
            Console.WriteLine("Playing {0} menu.", type);
            if (type == MenuType.Initial)
            {
                PhoneMenuCallController.SpeechSynthesizer.SpeakAsync(
                    "Press 1 for new orders, or press 2 for existing orders!");
            }
            else if (type == MenuType.Error)
            {
                PhoneMenuCallController.SpeechSynthesizer.SpeakAsync(
                    "Sorry, that is not a valid option. "
                    + "Press 1 to speak to an agent, or 2 to repeat this menu.");
            }
            else
            {
                PhoneMenuCallController.SpeechSynthesizer.SpeakAsync(
                    "Press 1 to speak to an agent, or 2 to repeat this menu.");
            }
        }

        void ToneController_ToneReceived(object sender, ToneControllerEventArgs e)
        {
            Console.WriteLine("Received tone {0}", e.Tone);

            PhoneMenuCallController.SpeechSynthesizer.SpeakAsyncCancelAll();

            PhoneMenuCallController.SpeechSynthesizer.Speak(String.Format(
                "You selected option {0}. Thank you for using the Phone Menu Sample App, goodbye!", e.Tone));

            //TODO: 3.2.8 - Terminate the call
            PhoneMenuCallController.AVCall.TerminateAsync();
        }

        void SpeechSynth_SpeakCompleted(object sender, SpeakCompletedEventArgs e)
        {
            Console.WriteLine("Speech completed");
        }
    }
}
