﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Rtc.Collaboration;
using Microsoft.Rtc.Signaling;
using Microsoft.Rtc.Collaboration.AudioVideo;
using System.Configuration;

namespace UCMACoreWorkflow
{
    public class PhoneMenuSample
    {
        CollaborationPlatform _platform;
        ApplicationEndpoint _endpoint;

        private static string _applicationId;
        private static string _applicationName;
        private static string _applicationEndpointURI;
        private static string _primaryLabUserId;

        //TODO: 3.2.1 - CallController singleton
        

        async internal void Start()
        {
            if (String.IsNullOrEmpty(ConfigurationManager.AppSettings["ApplicationId"]) ||
                (String.IsNullOrEmpty(ConfigurationManager.AppSettings["ApplicationName"])) ||
                (String.IsNullOrEmpty(ConfigurationManager.AppSettings["ApplicationEndpointURI"])) ||
                (String.IsNullOrEmpty(ConfigurationManager.AppSettings["PrimaryLabUserId"])))
            {
                throw new ArgumentNullException("Please provide values for ApplicationId, ApplicationName, ApplicationEndpointURI, and PrimaryLabUserId in app.config.");
            }

            _applicationId = ConfigurationManager.AppSettings["ApplicationId"];
            _applicationName = ConfigurationManager.AppSettings["ApplicationName"];
            _applicationEndpointURI = ConfigurationManager.AppSettings["ApplicationEndpointURI"];
            _primaryLabUserId = ConfigurationManager.AppSettings["PrimaryLabUserId"];

            var platformSettings = new ProvisionedApplicationPlatformSettings(_applicationName, _applicationId);
            _platform = new CollaborationPlatform(platformSettings);
            _platform.RegisterForApplicationEndpointSettings(OnApplicationEndpointSettingsDiscovered);

            try
            {
                await _platform.StartUpAsync();
                Console.WriteLine("Platform started.");
            }
            catch (InvalidOperationException ex)
            {
                Console.WriteLine(ex);
            }
            catch (RealTimeException ex)
            {
                Console.WriteLine(ex);
            }
        }

        async void OnApplicationEndpointSettingsDiscovered(object sender,
            ApplicationEndpointSettingsDiscoveredEventArgs e)
        {
            _endpoint = new ApplicationEndpoint(_platform, e.ApplicationEndpointSettings);
            _endpoint.RegisterForIncomingCall<AudioVideoCall>(OnAudioVideoCallReceived);

            try
            {
                await _endpoint.EstablishAsync();
                Console.WriteLine("Endpoint established.");
                Console.WriteLine("Place an audio call to {0} to continue.", _applicationEndpointURI);
            }
            catch (InvalidOperationException ex)
            {
                Console.WriteLine(ex);
            }
            catch (RealTimeException ex)
            {
                Console.WriteLine(ex);
            }
        }

        void OnAudioVideoCallReceived(object sender, CallReceivedEventArgs<AudioVideoCall> e)
        {
            //TODO: 3.2.2 - Handle incoming call 
            
        }

        async internal void Stop()
        {
            _endpoint.UnregisterForIncomingCall<AudioVideoCall>(OnAudioVideoCallReceived);

            try
            {
                await _endpoint.TerminateAsync();
                Console.WriteLine("Endpoint terminated.");

                await _platform.ShutDownAsync();
                Console.WriteLine("Platform shut down.");
            }
            catch (InvalidOperationException ex)
            {
                Console.WriteLine(ex);
            }
            catch (RealTimeException ex)
            {
                Console.WriteLine(ex);
            }
        }
    }

    public enum MenuType
    {
        Initial,
        Error,
        Repeat
    }
}
